/* GTK Napster-0.1 BETA 4
 * Copyright (C) 2000 Unified (xilliator@hotmail.com).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/time.h>

#include "common.h"
#include "nconnect.h"

gint napster_findbestserver(gtk_sess *sess, gchar *ip, gboolean *CANCEL, time_t *start_time)
{
	struct sockaddr_in dest_addr;
	gchar buffer[21];
	gint length = 0;
	gint sockcon = 0;
   fd_set readfds;
   struct timeval tv;
   gint old;
   gboolean LOCK;
	
	dest_addr.sin_family = AF_INET;
	dest_addr.sin_port = htons(PORT);
	dest_addr.sin_addr.s_addr = inet_addr(IP);
	memset(dest_addr.sin_zero, 0, 8);
	
	if((sockcon = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("socket");
  	  	gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_SOCKET);
		return FALSE;
	}
	
  	old = fcntl(sockcon, F_GETFL);
  	fcntl(sockcon, F_SETFL, O_NONBLOCK | old);

	*start_time = time(NULL);
  	while(connect(sockcon, (struct sockaddr *) &dest_addr, sizeof(struct sockaddr)) != 0)
  	{
		while(gtk_events_pending())
			gtk_main_iteration();
		if(*CANCEL)
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_CANCEL);
			close(sockcon);
			return FALSE;
		}
		if(difftime(time(NULL), *start_time) > sess->timeout)
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_TIMEOUT);
			close(sockcon);
			return FALSE;
		}
  	}

	fcntl(sockcon, F_SETFL, old);

	LOCK = TRUE;

	while(LOCK)
	{
		tv.tv_sec = 0;
		tv.tv_usec = 500000;

	  	FD_ZERO(&readfds);
	   FD_SET(sockcon, &readfds);
		
		if(select(sockcon+1, &readfds, NULL, NULL, &tv) == -1)
		{
			perror("select");
			close(sockcon);
			return FALSE;
		}
		
		if(FD_ISSET(sockcon, &readfds)) 
		{		
			length = recv(sockcon, buffer, 21, 0);
			LOCK = FALSE;
		}
		else
		{
			while(gtk_events_pending())
				gtk_main_iteration();
				
			if(*CANCEL)
			{
				gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_CANCEL);
				close(sockcon);
				return FALSE;
			}
			if(difftime(time(NULL), *start_time) > sess->timeout)
			{
				gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_TIMEOUT);
				close(sockcon);
				return FALSE;
			}
		}
	}

	close(sockcon);
	
	buffer[length] = '\0';
	
	strncpy(ip, buffer, length-6);
	ip[length-6] = '\0';
	return atoi(&buffer[length-5]);
}

gboolean napster_connect_to_server(gtk_sess *sess, gchar *ip, gint port, gboolean newuser, gboolean *CANCEL, time_t *start_time)
{
	struct sockaddr_in dest_addr;
	gchar *msg;
	gint old;
	gint16 length = 0;
	gint16 code = 0;
	fd_set readfds;
   struct timeval tv;
   gboolean LOCK;
	
	dest_addr.sin_family = AF_INET;
	dest_addr.sin_port = htons(port);
	dest_addr.sin_addr.s_addr = inet_addr(ip);
	memset(dest_addr.sin_zero, 0, 8);

	if((sess->sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("socket");
  	  	gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_SOCKET);
		return FALSE;
	}
	
	old = fcntl(sess->sockfd, F_GETFL);
  	fcntl(sess->sockfd, F_SETFL, O_NONBLOCK | old);

  	while(connect(sess->sockfd, (struct sockaddr *) &dest_addr, sizeof(struct sockaddr)) != 0)
  	{
		while(gtk_events_pending())
			gtk_main_iteration();
		if(*CANCEL)
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_CONNECT_CANCEL);
			close(sess->sockfd);
			return FALSE;
		}
		if(difftime(time(NULL), *start_time) > sess->timeout)
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_CONNECT_TIMOUT);
			close(sess->sockfd);
			return FALSE;
		}
  	}
	
	fcntl(sess->sockfd, F_SETFL, old);
	
	if(newuser)
	{
		if(!send_string(sess, (gchar *)sess->username, 7))
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_USERNAME);
			close(sess->sockfd);
			return FALSE;
		}

		while(gtk_events_pending())
			gtk_main_iteration();

		msg = (gchar *)g_malloc(145 + strlen((gchar *)LOGIN_ERROR_USERINUSE) + strlen((gchar *)VERISON_INFO_SEND));

		LOCK = TRUE;
		while(LOCK)
		{
			tv.tv_sec = 0;
			tv.tv_usec = 500000;

		  	FD_ZERO(&readfds);
		   FD_SET(sess->sockfd, &readfds);

			if(select(sess->sockfd+1, &readfds, NULL, NULL, &tv) == -1)
			{
				perror("select");
				close(sess->sockfd);
				g_free(msg);
				return FALSE;
			}

			if(FD_ISSET(sess->sockfd, &readfds)) 
			{		
				recv(sess->sockfd, msg, 6, 0);
				LOCK = FALSE;
			}
			else
			{
				while(gtk_events_pending())
					gtk_main_iteration();
				
				if(*CANCEL)
				{
					gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_VER_CANCEL);
					close(sess->sockfd);
					g_free(msg);
					return FALSE;
				}
				if(difftime(time(NULL), *start_time) > sess->timeout)
				{
					gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_VER_TIMEOUT);
					close(sess->sockfd);
					g_free(msg);
					return FALSE;
				}
			}
		}

		if(msg[2] == 9)
		{
			sprintf(msg, LOGIN_ERROR_USERINUSE, (gchar *)sess->username);
			gtk_label_set_text(GTK_LABEL(sess->login_label), msg);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		}
				
		while(gtk_events_pending())
			gtk_main_iteration();

		sprintf(msg, "%s %s 0 \"%s\" %d %s", (gchar *)sess->username, (gchar *)sess->password, (gchar *)VERISON_INFO_SEND, sess->speed, sess->email);
		
		if(!send_string(sess, msg, 6))
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_USERNAME_PASSWORD);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		}
	}
	else
	{
		msg = (gchar *)g_malloc(strlen((gchar *)sess->username) + strlen((gchar *)sess->password) + 22);
		sprintf(msg, "%s %s 0 \"%s\" %d", (gchar *)sess->username, (gchar *)sess->password, (gchar *)VERISON_INFO_SEND, sess->speed);

		if(!send_string(sess, msg, 2))
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_USERNAME_PASSWORD);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		}
 	}
 	
	gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_PASSWORD);

	LOCK = TRUE;

 	while(LOCK)
	{
		tv.tv_sec = 0;
		tv.tv_usec = 500000;

	  	FD_ZERO(&readfds);
	   FD_SET(sess->sockfd, &readfds);

		if(select(sess->sockfd+1, &readfds, NULL, NULL, &tv) == -1)
		{
			perror("select");
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		}
		
		if(FD_ISSET(sess->sockfd, &readfds)) 
		{		
			recv(sess->sockfd, msg, 4, 0);
			memcpy(&length, msg, 2);
			memcpy(&code, &msg[2], 2);
			msg = (gchar *)g_realloc(msg, length + 1);
			recv(sess->sockfd, msg, length, 0);
			LOCK = FALSE;
		}
		else
		{
			while(gtk_events_pending())
				gtk_main_iteration();
			
			if(*CANCEL)
			{
				gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_CANCEL);
				close(sess->sockfd);
				g_free(msg);
				return FALSE;
			}
			if(difftime(time(NULL), *start_time) > sess->timeout)
			{
				gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_TIMEOUT);
				close(sess->sockfd);
				g_free(msg);
				return FALSE;
			}
		}
	}

	if(code != 3)
	{
		gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_WRONG);
		close(sess->sockfd);
		g_free(msg);
		return FALSE;
	}
	
	g_free(msg);
	hl_send_logon_request(NULL, (gpointer)sess);
	sess->sockid = gdk_input_add(sess->sockfd, GDK_INPUT_READ, GTK_SIGNAL_FUNC(napster_something_to_read), sess);	
	return TRUE;
}

gboolean napster_connect(gtk_sess *sess, gboolean newuser, gboolean *LOCK, gboolean *CANCEL)
{
	gchar bestip[16];
  	gint bestport = 0;
  	time_t start_time;
	
	sess->CONNECTED = TRUE;
	*LOCK = TRUE;

	bestport = napster_findbestserver(sess, bestip, CANCEL, &start_time);

  	if(bestport != FALSE) 
  	{
  		gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_CONNECTING);

		while(gtk_events_pending())
			gtk_main_iteration();

		if(napster_connect_to_server(sess, bestip, bestport, newuser, CANCEL, &start_time))
  		{
			make_chat_box(sess, PRIVATE_CHAT_WINDOW, FALSE);
			*LOCK = FALSE;
			*CANCEL = FALSE;
			return TRUE;
		}
		else 
		{
			sess->CONNECTED = FALSE;
			*LOCK = FALSE;
			*CANCEL = FALSE;
			return FALSE;
		}
	}
	else 
	{
		sess->CONNECTED = FALSE;
		*LOCK = FALSE;
		*CANCEL = FALSE;
		return FALSE;
	}
}
